create database meddream;

\c meddream;

CREATE TABLE privilege (
	username varchar(64) NOT NULL PRIMARY KEY,
		-- database login username. This user must have at least the SELECT privilege on this table
	firstname varchar(64) NOT NULL,
		-- operator's first name for a certain search filter applied when .viewprivate=0 (PHP only)
	lastname varchar(64) NOT NULL,
		-- operator's last name for a certain search filter applied when .viewprivate=0 (PHP only)
	viewprivate smallint DEFAULT '0',
		-- operator can view studies with study.private=1. If .viewprivate=0, then Search lists only studies that match the abovementioned filter (PHP only; no such restrictions in Java so far)
	forward smallint DEFAULT '0',
		-- operator can use the "Forward" function
	export smallint DEFAULT '0',
		-- operator can use the "Export" function
	upload smallint DEFAULT '0',
		-- operator has write access in the "Reporting" function. If .upload=0, then the access is read only
	admin smallint DEFAULT '0',
		-- operator can access MedDream settings
	usergroup varchar(64) DEFAULT NULL
		-- value '_ADMIN' is a synonym for .admin='1'
);

CREATE TABLE patient (
	origid varchar(64) NOT NULL DEFAULT '' PRIMARY KEY,
		-- (0010,0020) Patient Id. Displayed in search results
	lastname varchar(64) NOT NULL DEFAULT '',
		-- (0010,0010) Patient's Name, 1st component in 1st group. Displayed in search results
	firstname varchar(64) NOT NULL DEFAULT '',
		-- (0010,0010) Patient's Name, 2nd component in 1st group. Displayed in search results
	middlename varchar(64) DEFAULT NULL,
		-- (0010,0010) Patient's Name, 3rd component in 1st group. Supported only by "Save Annotations" function
	prefix varchar(64) DEFAULT NULL,
		-- (0010,0010) Patient's Name, 4th component in 1st group. Supported only by "Save Annotations" function
	suffix varchar(64) DEFAULT NULL,
		-- (0010,0010) Patient's Name, 5th component in 1st group. Supported only by "Save Annotations" function
	ideographic varchar(64) DEFAULT NULL,
		-- (0010,0010) Patient's Name, entire 2nd group. Supported only by "Save Annotations" function
	phonetic varchar(64) DEFAULT NULL,
		-- (0010,0010) Patient's Name, entire 3rd group. Supported only by "Save Annotations" function
	birthdate date DEFAULT NULL,
		-- (0010,0030) Patient's Birth Date. Needed by "Reporting" function
	sex varchar(16) DEFAULT NULL,
		-- (0010,0040) Patient's Sex. Needed by "Reporting" function
	history text
		-- (0010,21b0) Additional Patient History. Needed by "Reporting" function
);
CREATE INDEX fullname ON patient (lastname,firstname);

CREATE TABLE study (
	uuid varchar(64) NOT NULL PRIMARY KEY,
		-- (0020,000d) Study Instance UID
	id varchar(16) DEFAULT NULL,
		-- (0020,0010) Study ID. Displayed in search results
	patientid varchar(64) NOT NULL DEFAULT '' REFERENCES patient (origid),
		-- foreign key to patient.origid
	studydate date DEFAULT NULL,
		-- (0008,0020) Study Date. Displayed in search results
	studytime time DEFAULT NULL,
		-- (0008,0030) Study Time. Displayed in search results
	accessionnum varchar(16) DEFAULT NULL,
		-- (0008,0050) Accession Number. Displayed in search results
	modalities varchar(16) DEFAULT NULL,
		-- distinct values from series.modality, separated by backslash. NULL is OK
	referringphysician varchar(64) DEFAULT NULL,
		-- (0008,0090) Referring Physician's Name. Can be used by a search filter, otherwise NULL is OK
	description varchar(64) DEFAULT NULL,
		-- (0008,1030) Study Description. Displayed in search results
	readingphysician varchar(64) DEFAULT NULL,
		-- (0008,1060) Name of Physician(s) Reading Study. Can be used by a search filter, otherwise NULL is OK
	private smallint DEFAULT '1',
		-- leave nonzero
	received timestamp DEFAULT NULL,
		-- when study was sent to the PACS. Displayed in search results
	sourceae varchar(64) DEFAULT NULL,
		-- Source AE Title of the modality that sent the study to the PACS. Displayed in search results
	reviewed varchar(64) DEFAULT NULL
		-- login name of the user who last viewed the study. If empty, the search result is displayed in bold which suggests "unread" state
);
CREATE INDEX id ON study (id);
CREATE INDEX patientid ON study (patientid);
CREATE INDEX accessionnum ON study (accessionnum);
CREATE INDEX received ON study (received);
CREATE INDEX studydate ON study (studydate);

CREATE TABLE series (
	uuid varchar(64) NOT NULL PRIMARY KEY,
		-- (0020,000e) Series Instance UID
	studyuid varchar(64) NOT NULL REFERENCES study (uuid),
		-- foreign key to study.uuid
	description varchar(64) DEFAULT NULL,
		-- (0008,103e) Series Description. Visible in the thumbnail pane
	modality varchar(16) DEFAULT NULL,
		-- (0008,0060) Modality. Important for display of the thumbnail pane
	seriesnumber integer DEFAULT NULL
		-- (0020,0011) Series Number. Needed to sort series in the study
);
CREATE INDEX studyuid ON series (studyuid);


CREATE TABLE image (
	uuid varchar(64) NOT NULL PRIMARY KEY,
		-- (0008,0018) SOP Instance UID
	sopclass varchar(64) NOT NULL,
		-- (0008,0016) SOP Class UID. Important for display of the thumbnail pane
	seriesuid varchar(64) NOT NULL REFERENCES series (uuid),
		-- foreign key to series.uuid
	xfersyntax varchar(64) NOT NULL DEFAULT '1.2.840.10008.1.2',
		-- (0002,0010) Transfer Syntax UID. Without it, some images might load slower or look worse
	instance integer DEFAULT NULL,
		-- (0020,0013) Instance Number. Needed to sort images in the series
	numrows smallint DEFAULT NULL,
		-- (0028,0010) Rows. Only for the "Save Annotations" function, otherwise NULL is OK
	numcolumns smallint DEFAULT NULL,
		-- (0028,0011) Columns. Only for the "Save Annotations" function, otherwise NULL is OK
	bitsstored smallint DEFAULT NULL,
		-- (0028,0101) Bits Stored. Legacy parameter, NULL is OK
	path varchar(255) NOT NULL,
		-- full path to DICOM file
	numframes smallint DEFAULT NULL
		-- (0028,0008) Number Of Frames. Important for display of the thumbnail pane
);
CREATE INDEX seriesuid ON image (seriesuid);

CREATE USER meddream WITH encrypted PASSWORD :pass;
GRANT SELECT ON TABLE privilege,patient,study,series,image TO meddream;
